unit EanRB;

interface

uses Classes, Windows, SysUtils,Graphics, Messages,
     Controls,EanSpecs,EanKod,
     ppCtrls, ppDsgnCt, ppDevice, ppDrwCmd, ppTypes, ppClass;

{$I EAN.INC}

type

  TRBCustomEan = class(TppCustomComponent)
     private
         FCharWidthMM : Double;
         FOnBeforePrint,
         FOnAfterPrint : TNotifyEvent;
         procedure   SetBarCode(Value:String);
         procedure   SetShowLabels(Value:Boolean);
         procedure   SetTransparent(Value:Boolean);
         procedure   SetStartStopLines(Value:Boolean);
         procedure   SetLinesColor(Value:TColor);
         procedure   SetBackgroundColor(Value:TColor);
         procedure   SetSecurity(Value:Boolean);
         procedure   SetDemoVersion(Value:Boolean);
         procedure   SetEan13AddUp(Value:Boolean);
         procedure   SetFontAutoSize(Value:Boolean);
         Function    GetBarCode       :String;
         Function    GetShowLabels    :Boolean;
         Function    GetTransparent   :Boolean;
         Function    GetStartStopLines:Boolean;
         Function    GetLinesColor    :TColor;
         Function    GetBackgroundColor :TColor;
         Function    GetEan13AddUp      :Boolean;
         Function    GetFontAutoSize    :Boolean;
         Function    GetSecurity        :Boolean;
         Function    GetDemoVersion     :Boolean;
         Function    GetTypBarCode      :TTypBarCode;
         Function    GetLAstPaintError  :TLastPaintError;
         Function    GetAngle           :integer;
         Procedure   SetAngle(Value:Integer);
         Function    GetLabelMask:String;
         Procedure   SetLabelMask(Value:String);
         Procedure   SetCharWidthMM(Value:Double);
         function    GetCaption:TBarcodeCaption;
         function    GetCaptionBottom:TBarcodeCaption;
         Procedure   SetAutoSize(Value:Boolean);
         Function    GetAutoSize:Boolean;
         Procedure   SetDisableEditor(Value:Boolean);
         Function    GetDisableEditor:Boolean;
         procedure   RekalkWidthMM;
     protected
            FEan : TCustomEan;
            constructor Create(AOwner:TComponent); override;
            destructor  Destroy;                   override;
            procedure   SetTypBarCode(Value:TTypBarCode);   virtual;
            property    DemoVersion : Boolean Read GetDemoVersion Write SetDemoVersion;
            property    OnBeforePrint : TNotifyEvent Read FOnBeforePrint Write FOnBeforePrint;
            property    OnAfterPrint  : TNotifyEvent Read FOnAfterPrint  Write FOnAfterPrint;

            procedure CreatePopupMenu(aOwner: TComponent; var aPopupMenu: TppPopupMenu); override;
            procedure PaintDesignControl(aCanvas: TCanvas); override;
            procedure PropertiesToDrawCommand(aDrawCommand: TppDrawCommand); override;
            procedure Invalidate;
            procedure PopupMenuClick(Sender: TObject); override;
            procedure SettingsMenuClick(Sender: TObject);
            function  GetDefaultPropHint: string; Override;
            procedure GetDefaultPropEnumNames(aList: TStrings); override;
     public
         procedure  Next;
         function   GetSetOfChars:string;
         function   GetSetOfCharsVisible:String;
         function   CheckBarCode(var S:String):Boolean;
         function   LastPaintErrorText:String;
         procedure  ActiveSetupWindow;
         procedure  Copyright;
         function   DigitVisible(idx:integer):Boolean;
         function   Ean : TCustomEan;
         procedure  Loaded;          override;
         property BackgroundColor   : TColor         Read GetBackgroundColor Write SetBackgroundColor;
         property Transparent       : Boolean        Read GetTransparent     Write SetTransparent;
         property ShowLabels        : Boolean        Read GetShowLabels      Write SetShowLabels;
         property StartStopLines    : Boolean        Read GetStartStopLines  Write SetStartStopLines;
         property TypBarCode        : TTypBarCode    Read GetTypBarCode      Write SetTypBarCode;
         property LinesColor        : TColor         Read GetLinesColor      Write SetLinesColor;
         property Ean13AddUp        : Boolean        Read GetEan13AddUp      Write SetEan13AddUp;
         property FontAutoSize      : Boolean        Read GetFontAutoSize    Write SetFontAutoSize;
         property Security          : Boolean        Read GetSecurity        Write SetSecurity;
         property Font;
         property BarCode           : string          Read GetBarCode         Write SetBarCode;
         property LastPaintError    : TLastPaintError Read GetLastPaintError;
         property Angle             : Integer         Read GetAngle           Write SetAngle;
         property LabelMask         : string          Read GetLabelMask       Write SetLabelMask;
         property CharWidthMM       : Double          Read FCharWidthMM       Write SetCharWidthMM;
         property Caption           : TBarcodeCaption Read GetCaption;
         property CaptionBottom     : TBarcodeCaption Read GetCaptionBottom;
         property AutoSize          : Boolean         Read GetAutoSize        Write SetAutoSize;
         property DisableEditor     : Boolean         Read GetDisableEditor   Write SetDisableEditor;
     published
         property Visible;
     end;

  TRBEan   = class(TRBCustomEan)
     published
         property BackgroundColor;
         property Transparent;
         property ShowLabels;
         property StartStopLines;
         property TypBarCode;
         property LinesColor;
         property Ean13AddUp;
         property FontAutoSize;
         property Security;
         property DemoVersion;
         property Font;
         property LastPaintError;
         property BarCode;
         property Angle;
         property LabelMask;
         property CharWidthMM;
         property Caption;
         property CaptionBottom;
         property AutoSize;
         property DisableEditor;
         // property Size;
         property OnBeforePrint;
         property OnAfterPrint;
         {events - inherited from TppPrintable}
         property OnDrawCommandClick;
         property OnDrawCommandCreate;
         property OnPrint;
  end;




  TRBEanMemo   = class(TRBCustomEan)
     private
         procedure   SetTypBarCode(Value:TTypBarCode); override;
     protected
         function  GetLines:TStringList;
         procedure SetLines(Value:TStringList);
     public
         constructor Create(AOwner: TComponent); override;
         destructor Destroy; override;
     published
         property Lines:TStringList Read GetLines Write SetLines;
         property BackgroundColor;
         property Transparent;
         property ShowLabels;
         property StartStopLines;
         property TypBarCode;
         property LinesColor;
         property Ean13AddUp;
         property FontAutoSize;
         property Security;
         property DemoVersion;
         property Font;
         property LastPaintError;
         property BarCode;
         property Angle;
         property LabelMask;
         property CharWidthMM;
         property Caption;
         property CaptionBottom;
         property AutoSize;
         property DisableEditor;
         // property Size;
         property OnBeforePrint;
         property OnAfterPrint;
     end;


implementation
uses Forms, EanFmt3 ;

constructor TRBCustomEan.Create(AOwner:TComponent);
var i:Integer;
begin
     inherited Create(AOwner);
     FEan          := TEan.Create(self);
     FEan.Security := False;
     i:=FEan.MinWidth;
     if Width<i then Width:=i;

     Height       := FEan.Height;
     Width        := FEan.Width;
     FCharWidthMM := 0;


     // properties needed by Report Builder
     DrawCommandClass    := TppDrawImage;
     DefaultPropName     := 'TypBarCode';
     DefaultPropEditType := etValueList;

     spHeight := FEan.Height;
     spWidth  := FEan.Width;

end;

destructor  TRBCustomEan.Destroy;
begin
     inherited Destroy;
end;

function TRBCustomEan.GetDefaultPropHint: String;
begin
  Result := 'Bar code types';
end;

{------------------------------------------------------------------------------}
procedure TRBCustomEan.GetDefaultPropEnumNames(aList: TStrings);
begin
  aList.Clear;
  FEan.AddTypesToList(aList, btSymbol);
end;


procedure TRBCustomEan.RekalkWidthMM;
var SizeMM : Double;
begin{
     if CharWidthMM<>0 then begin
           SizeMM:=CharWidthMM*Length(FEan.Barcode);

           case Self.ParentReport.Units of
                    Characters : ;
                    MM     : Size.Width := Round(SizeMM);
                    Inches : Size.Width := Round(SizeMM/25.4);
                    Native : Size.Width := Round(10*SizeMM);
                    Pixels : ;
           end;
     end;
     }
end;

procedure   TRBCustomEan.SetBarCode(Value:String);
begin
     if Printing then Exit;
     if FEan.BarCode<>Value then begin
        FEan.BarCode:=Value;
        RekalkWidthMM;
        Invalidate;
     end;
end;

procedure   TRBCustomEan.SetShowLabels(Value:Boolean);
begin
     if Printing then Exit;
     if FEan.ShowLabels<>Value then begin
        FEan.ShowLabels:=Value;
        Invalidate;
     end;
end;

procedure   TRBCustomEan.SetTransparent(Value:Boolean);
begin
     if Printing then Exit;
     if FEan.Transparent<>Value then begin
        FEan.Transparent:=Value;
        Invalidate;
     end;
end;

procedure   TRBCustomEan.SetStartStopLines(Value:Boolean);
begin
     if Printing then Exit;
     if FEan.StartStopLines<>Value then begin
        FEan.StartStopLines:=Value;
        Invalidate;
     end;
end;

procedure   TRBCustomEan.SetLinesColor(Value:TColor);
begin
     if Printing then Exit;
     if FEan.LinesColor<>Value then begin
        FEan.LinesColor:=Value;
        Invalidate;
     end;
end;

procedure   TRBCustomEan.SetBackgroundColor(Value:TColor);
begin
     if Printing then Exit;
     if FEan.BackgroundColor<>Value then begin
        FEan.BackgroundColor:=Value;
        Invalidate;
     end;
end;

procedure   TRBCustomEan.SetEan13AddUp(Value:Boolean);
begin
     if Printing then Exit;
     if FEan.Ean13AddUp<>Value then begin
        FEan.Ean13AddUp:=Value;
        Invalidate;
     end;
end;

procedure   TRBCustomEan.SetFontAutoSize(Value:Boolean);
begin
     if Printing then Exit;
     if FEan.FontAutoSize<>Value then begin
        FEan.FontAutoSize:=Value;
        Invalidate;
     end;
end;

procedure   TRBCustomEan.SetSecurity(Value:Boolean);
begin
     if Printing then Exit;
     if FEan.Security<>Value then begin
        FEan.Security:=Value;
        Invalidate;
     end;
end;

procedure   TRBCustomEan.SetDemoVersion(Value:Boolean);
begin
     if Printing then Exit;
     if FEan.DemoVersion<>Value then begin
        FEan.DemoVersion:=Value;
        Invalidate;
     end;
end;


procedure   TRBCustomEan.SetTypBarCode(Value:TTypBarCode);
var i:Integer;
begin
     if Printing then Exit;
     if FEan.TypBarCode<>Value then begin
        FEan.TypBarCode:=Value;
        RekalkWidthMM;
        i:=fEan.MinWidth;
        if Width<i then Width := i;
        Invalidate;
     end;
end;


Function    TRBCustomEan.GetBarCode       :String;
begin
     Result:=FEan.BarCode;
end;

Function    TRBCustomEan.GetShowLabels    :Boolean;
begin
     Result:=FEan.ShowLabels;
end;

Function    TRBCustomEan.GetTransparent   :Boolean;
begin
     Result:=FEan.Transparent;
end;

Function    TRBCustomEan.GetStartStopLines:Boolean;
begin
     Result:=FEan.StartStopLines;
end;

Function    TRBCustomEan.GetLinesColor    :TColor;
begin
     Result:=FEan.LinesColor;
end;

Function    TRBCustomEan.GetBackgroundColor :TColor;
begin
     Result:=FEan.BackgroundColor;
end;

Function    TRBCustomEan.GetEan13AddUp      :Boolean;
begin
     Result:=FEan.Ean13AddUp;
end;

Function    TRBCustomEan.GetFontAutoSize  :Boolean;
begin
     Result:=FEan.FontAutoSize;
end;

Function    TRBCustomEan.GetSecurity        :Boolean;
begin
     Result:=FEan.Security;
end;

Function    TRBCustomEan.GetDemoVersion:Boolean;
begin
     Result:=FEan.DemoVersion;
end;

Function    TRBCustomEan.GetTypBarCode      :TTypBarCode;
begin
     Result:=fEan.TypBarCode;
end;


Function    TRBCustomEan.GetLastPaintError  :TLastPaintError;
begin
     Result:=fEan.LastPaintError;
end;


procedure TRBCustomEan.Next;
begin
     FEan.Next;
end;

function   TRBCustomEan.GetSetOfChars:string;
begin
     Result:=FEan.GetSetOfChars;
end;

function   TRBCustomEan.GetSetOfCharsVisible:String;
begin
     Result:=FEan.GetSetOfCharsVisible;
end;

function   TRBCustomEan.CheckBarCode(var S:String):Boolean;
begin
     Result:=FEan.CheckBarCode(S);
end;


function   TRBCustomEan.Ean : TCustomEan;
begin
     Result := FEan;
end;

function   TRBCustomEan.LastPaintErrorText:String;
begin
     Result:=FEan.LastPaintErrorText;
end;

Function    TRBCustomEan.GetAngle:integer;
begin
     Result := FEan.Angle;
end;

Procedure   TRBCustomEan.SetAngle(Value:Integer);
begin
     FEan.Angle := Value;
     Invalidate;
end;

procedure TRBCustomEAN.ActiveSetupWindow;
begin
   if not (Parent is TEanSetupFmt) then
     with TEanSetupFmt.Create(Application) do
       Try
          SetParentEAN(FEan);
          if ShowModal=mrOK then
             FEan.Assign(EAN);
       Finally
          Free;
       End;
end;


procedure  TRBCustomEAN.Copyright;
begin
     with TEanSetupFmt.Create(Application) do
       Try
          SetParentEAN(FEan);
          ZAL.ActivePage := SH_COPYRIGHT;
          if ShowModal=mrOK then
             FEan.Assign(EAN);
       Finally
          Free;
       End;
end;


function TRBCustomEAN.DigitVisible(idx:integer):Boolean;
begin
     Result := True;
     if Length(LabelMask)>=idx then
        if LabelMask[idx]<>'_' then
        Result:=False;
end;

Function    TRBCustomEan.GetLabelMask:String;
begin
     Result := FEan.LabelMask;
end;


Procedure   TRBCustomEan.SetLabelMask(Value:String);
begin
     FEan.LabelMask := Value;
end;

Procedure   TRBCustomEan.SetCharWidthMM(Value:Double);
var s:String;
begin
     FCharWidthMM := Value;
     RekalkWidthMM;
end;

function    TRBCustomEan.GetCaption:TBarcodeCaption;
begin
     Result := FEan.Caption;
end;

function    TRBCustomEan.GetCaptionBottom:TBarcodeCaption;
begin
     Result := FEan.CaptionBottom;
end;

Procedure   TRBCustomEan.SetAutoSize(Value:Boolean);
var i:Integer;
begin
     FEan.AutoSize := Value;
     i:=FEan.MinWidth;
     if Width<i then
        Width := i;
end;

Function    TRBCustomEan.GetAutoSize:Boolean;
begin
     Result := FEan.AutoSize;
end;

Procedure   TRBCustomEan.SetDisableEditor(Value:Boolean);
begin
     FEan.DisableEditor:=Value;
end;

Function    TRBCustomEan.GetDisableEditor:Boolean;
begin
     Result := FEan.DisableEditor;
end;

procedure  TRBCustomEan.Loaded;
begin
     inherited Loaded;
     RekalkWidthMM;
end;

procedure TRBCustomEan.Invalidate;
begin
  {notify report designer}
  PropertyChange;
  {notify report engine}
  Reset;
  {redraw design control}
  InvalidateDesignControl;
end;


procedure TRBCustomEan.CreatePopupMenu(aOwner: TComponent; var aPopupMenu: TppPopupMenu);
begin
  inherited CreatePopupMenu(aOwner, aPopupMenu);
  aPopupMenu.AddItem(80, 'Line2', '-',  0);
  aPopupMenu.AddItem(90, 'Settings', 'Settings',  0);
  // aPopupMenu.ItemByName('Settings').OnClick := SettingsMenuClick;
end;

procedure TRBCustomEan.PopupMenuClick(Sender: TObject);
begin
  TppPopupMenu(Sender).ItemByName('Settings').OnClick := SettingsMenuClick;
end;

procedure TRBCustomEan.PaintDesignControl(aCanvas: TCanvas);
begin
     PaintBarCode(aCanvas,Rect(0,0,Round(spWidth),Round(spHeight)), FEan);
end;

procedure TRBCustomEan.PropertiesToDrawCommand(aDrawCommand: TppDrawCommand);
var lDrawImage : TppDrawImage;
    Bitmap     : TBitmap;
begin
  inherited PropertiesToDrawCommand(aDrawCommand);
  if not(aDrawCommand is TppDrawText) then Exit;

  lDrawImage := TppDrawImage(aDrawCommand);
  lDrawImage.Left    := PrintPosRect.Left;
  lDrawImage.Top     := PrintPosRect.Top;
  lDrawImage.Height  := PrintPosRect.Bottom - PrintPosRect.Top;
  lDrawImage.Width   := PrintPosRect.Right - PrintPosRect.Left;

  Bitmap := TBitmap.Create;
  try
         Bitmap.PixelFormat := pf4Bit;
         Bitmap.Height := lDrawImage.Height;
         Bitmap.Width  := lDrawImage.Width;
         PaintBarCode(Bitmap.Canvas,Rect(0,0,lDrawImage.Width,lDrawImage.Height),FEan);
         lDrawImage.Picture.Assign(Bitmap);
  finally
         Bitmap.Free;
  end;
end;

procedure TRBCustomEan.SettingsMenuClick(Sender: TObject);
begin
     FEan.ActiveSetupWindow;
     Invalidate;
end;

constructor TRBEanMemo.Create(AOwner: TComponent);
begin
     inherited Create(AOwner);
     FEan.Free;
     FEan:=TEanMemo.Create(Self);
     ShowLabels     := False;
     StartStopLines := False;
end;

Destructor TRBEanMemo.Destroy;
begin
     inherited Destroy;
end;

procedure TRBEanMemo.SetLines(Value:TStringList);
begin
    TEanMemo(FEan).Lines.Assign(Value);
    Invalidate;
end;

procedure   TRBEanMemo.SetTypBarCode(Value:TTypBarCode);
begin
     if FEan.TypBarCode<>Value then begin
        inherited SetTypBarCode(Value);
        TEanMemo(FEan).Lines.Clear;
        TEanMemo(FEan).Lines.Add(BarCode);
     end;
end;

function TRBEanMemo.GetLines:TStringList;
begin
     Result:=TEanMemo(FEan).Lines;
end;




initialization
  ppRegisterComponent(TRBEan,     'Ean components', 10, 0, 'RbEan', HInstance);
  ppRegisterComponent(TRBEanMemo, 'Ean components', 10, 0, 'RbEanMemo', HInstance);

finalization

  ppUnRegisterComponent(TRbEan);
  ppUnRegisterComponent(TRbEanMemo);

end.
